/*
	file automatically generated by make_test_files.pl
	Fri Aug  6 16:50:44 2004
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for HINV
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_hinv_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/



/* prototypes */

#define COMPARE_SAMPLE_SIZE   (500)
#define VIOLATE_SAMPLE_SIZE   (20)

#ifndef M_PI
#define M_PI       3.14159265358979323846264338328      /* pi */
#endif

double sin_cdf( double x, const UNUR_DISTR *distr );
double sin_pdf( double x, const UNUR_DISTR *distr );
double sin_dpdf( double x, const UNUR_DISTR *distr );

double sin0_cdf( double x, const UNUR_DISTR *distr );
double sin0_pdf( double x, const UNUR_DISTR *distr );
double sin0_dpdf( double x, const UNUR_DISTR *distr );




/*---------------------------------------------------------------------------*/
/* [verbatim] */



/*---------------------------------------------------------------------------*/
/* Example of  a distribution with f(x)=0 at x=-0.25 and x=0.75              */
/*                                                                           */
/*          /  (1 +sin(2 Pi x))/2  if |x| <= 1                               */
/*  f(x) = <                                                                 */
/*          \  0        otherwise                                            */
/*                                                                           */
double sin_pdf( double x, const UNUR_DISTR *distr )
{
  if (fabs(x) > 1.)
    return 0.;
  else
    return (0.5*(1.+sin((2.*M_PI)*x)));
} /* end of sin_pdf() */

/* The derivative of the PDF of our distribution:                */
double sin_dpdf( double x, const UNUR_DISTR *distr )
{
  if (fabs(x) > 1.)
    return 0.;
  else
    return (M_PI*cos((2.*M_PI)*x));
} /* end of sin_dpdf() */

/* The CDF of our distribution:                                  */
double sin_cdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.)
    return 0.;
  else if(x<=1.)
    return (1.+2.*M_PI*(1+x)-cos((2.*M_PI)*x))/(4.*M_PI);
  else return 1.;
} /* end of sin_cdf() */

/*---------------------------------------------------------------------------*/

/* Example of  a distribution with f(x)=0 in (-0.5,0)                        */
/*                                                                           */
/*          /  Max(sin(2 Pi x)),0)Pi/2  if -1 < x <0.5                       */
/*  f(x) = <                                                                 */
/*          \  0        otherwise                                            */
/*                                                                           */
double sin0_pdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.) return 0.;
  if (x <= -0.5) return sin((2.*M_PI)*x)*0.5*M_PI;
  if (x < 0.) return 0.;
  if (x <= 0.5) return sin((2.*M_PI)*x)*0.5*M_PI;
  return 0.;
} /* end of sin0_pdf() */

/* The derivative of the PDF of our distribution:                */
double sin0_dpdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.) return 0.;
  if (x <= -0.5) return cos((2.*M_PI)*x)*M_PI*M_PI;
  if (x < 0.) return 0.;
  if (x <= 0.5) return cos((2.*M_PI)*x)*M_PI*M_PI;
  return 0.;
} /* end of sin0_dpdf() */

/* The CDF of our distribution:                                  */
double sin0_cdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.) return 0.;
  if(x<= -0.5) return 0.25*(1-cos((2.*M_PI)*x));
  if (x < 0.) return 0.5;
  if (x <= 0.5) return 0.75-0.25*cos((2.*M_PI)*x);
  return 1.;
} /* end of sin0_cdf() */



/*****************************************************************************/

int
hinv_error_gen_experiment( UNUR_GEN *gen,
			   int samplesize,    /* n for error experiment      */
			   int output,        /* 0-no,1- little, 2-more      */
			   int order,
			   double eu )
     /* returns 0 if MAE > eu, 1 if maxerror>eu  otherwise 2                 */
{   
  double maxerror, MAE;
  int i, nfpar;
  const double *fpar;     
  const UNUR_DISTR *distr = unur_get_distr(gen);
  const char *genid = unur_get_genid(gen);

  unur_hinv_estimate_error(gen, samplesize, &maxerror, &MAE);

  if((output==1 && MAE > eu)||output==2) {
    fprintf(TESTLOG,"%s: %s distribution",genid,unur_distr_get_name(distr));
    nfpar = unur_distr_cont_get_pdfparams(distr,&fpar);
    if (nfpar) { 
      fprintf(TESTLOG," with parameters ");
      for(i=0;i<nfpar;i++) fprintf(TESTLOG," | %g",fpar[i]);
      fprintf(TESTLOG," |");
    }
    fprintf(TESTLOG,":\n");

    fprintf(TESTLOG,"%s:   setting: order = %d, u-error <= %g, samplesize = %d\n",genid,order,eu,samplesize);

    fprintf(TESTLOG,"%s:   result: # intervals = %d, ",genid,unur_hinv_get_n_intervals(gen));
    fprintf(TESTLOG,"max u-error = %g, MAE = %g  ",maxerror,MAE);
    if(maxerror > eu) {
      fprintf(TESTLOG,"(NOT <= %g)\n",eu);
      fprintf(TESTLOG,"%s: Warning: precision problem, maxerror > u_resolution\n",genid);
    }
    else {
      fprintf(TESTLOG,"(<= %g)\n",eu);
    }
  } 

  if(MAE > eu) {
    printf("(!+)");
    return 0;
  } else if(maxerror > eu) {
    printf("(+?)");
    return 1;
  } else {
    printf("+");
    return 2;
  }
} /* end of hinv_error_gen_experiment() */

/*****************************************************************************/

int
hinv_error_experiment( UNUR_DISTR *distr,
		       double *cpoints,   /* pointer to c-point array        */
		       int ncp,           /* number of c-points              */
		       double eu,         /* epsilon-u                       */
		       int order,         /* order of polynomial             */
		       int samplesize,    /* n for error experiment          */
		       int output)        /* 0-no,1- little, 2-more          */
     /* returns 0 if MAE > eu, 1 if maxerror>eu  otherwise 2 */

{ 
  int i, nfpar;
  const double *fpar;     
  UNUR_PAR   *par;      /* parameter object                      */
  UNUR_GEN   *gen;      /* generator object                      */

  nfpar = unur_distr_cont_get_pdfparams(distr,&fpar);
  par = unur_hinv_new(distr);
  unur_hinv_set_order(par,order);
  if(cpoints != 0 && ncp>0) unur_hinv_set_cpoints(par, cpoints, ncp);
  unur_hinv_set_u_resolution(par,eu);
  unur_set_debug(par,1u);
  gen = unur_init(par);

  i = hinv_error_gen_experiment(gen, samplesize, output, order, eu);

  unur_free(gen);

  return i;
} /* end of hinv_error_experiment() */

/*****************************************************************************/

int
check_domain( UNUR_GEN   *gen,
	      int samplesize,
	      double left,
	      double right,
	      int output )
     /* returns 0 if samples outside of left right were generated, 
	1 if samples were too far away from border,
	otherwise 2
     */
{ 
  int i, retvalue;
  double x, min, max, cdfleft,cdfright,cdfmin,cdfmax;
  const UNUR_DISTR *distr = unur_get_distr(gen);
  const char *genid = unur_get_genid(gen);

  min = 1.e99;
  max = -1.e99;
  for(i=0; i<samplesize; i++){
    x = unur_sample_cont(gen);
    if(x < min) min = x;
    if(x > max) max = x;
  }
  cdfmin = unur_distr_cont_eval_cdf(min,distr);
  cdfleft = unur_distr_cont_eval_cdf(left,distr); 
  cdfmax = unur_distr_cont_eval_cdf(max,distr);
  cdfright = unur_distr_cont_eval_cdf(right,distr); 
  
  if(min < left || max > right) retvalue = 0;
  if(cdfmin-cdfleft > 100./samplesize
     ||cdfright - cdfmax > 100./samplesize )
    retvalue = 1;
  else retvalue = 2;
  
  if(output==2 || (output==1 && retvalue<=1)){   
    if(retvalue<=1) fprintf(TESTLOG,"%s: ERROR with domain:\n",genid);
    fprintf(TESTLOG,"%s:   left=%g, min=%g, max=%g, right=%g\n",genid,left,min,max,right);
    fprintf(TESTLOG,"%s:   cdfleft=%g, cdfmin=%g, cdfmax=%g, cdfright=%g\n\n",genid,cdfleft,cdfmin,cdfmax,cdfright);
  }
  
  if(retvalue == 2) printf("+");
  if(retvalue == 1) printf("(+?)");
  if(retvalue == 0) printf("!+");
  
  return retvalue;
} /* check_domain() */

/*****************************************************************************/

int
chg_domain_experiment( UNUR_GEN *gen, 
		       double eu,
		       int order,
		       int samplesize,
		       int output,
		       double left,   /* border of userdefined original domain */
		       double right ) /* border of userdefined original domain */
     /* returns 0 if MAE > eu or a domainerror occured,
	1 if maxerror>eu occured,
	2 otherwise 
     */
{ 
  int i,k,mini=9999;
  double newleft, newright;

  for(k=1; k<=10;k++){
    newleft = left + k*0.05*(right-left);
    newright = right - k*0.04*(right-left);
    unur_hinv_chg_truncated(gen, newleft, newright);
    i = hinv_error_gen_experiment(gen, samplesize/10, output, order, eu);
    if(i < mini) mini = i;
    i = check_domain(gen,samplesize,newleft,newright,output);
    if(i < mini) mini = i;
    unur_hinv_chg_truncated(gen, left, right);
    i = hinv_error_gen_experiment(gen, samplesize/10, output, order, eu);
    if(i < mini) mini = i;
    i = check_domain(gen,samplesize,left,right,output);
    if(i < mini) mini = i;
  }
  
  return mini;
} /* end of chg_domain_experiment() */

/*****************************************************************************/
/*---------------------------------------------------------------------------*/
/* [new] */

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_DISTR *distr = NULL;
   distr = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,37,(unur_hinv_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,37,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* invalid distribution type */
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,43,(unur_hinv_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,43,UNUR_ERR_DISTR_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* data missing in distribution object */
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 


unur_errno = 0;
/* cdf */
n_tests_failed += (check_expected_NULL(TESTLOG,50,(unur_hinv_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,50,UNUR_ERR_DISTR_REQUIRED)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/
/* [set] */

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_PAR   *par = NULL;
   double stp[] = {1.,2.,3.};
   par = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,61,(unur_hinv_set_order( par, 3 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,61,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,64,(unur_hinv_set_u_resolution( par, 1.e-8 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,64,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,67,(unur_hinv_set_cpoints(par,stp,3)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,67,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,70,(unur_hinv_set_boundary(par,1.,3.)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,70,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,73,(unur_hinv_set_guidefactor( par, 2. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,73,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,76,(unur_hinv_set_max_intervals(par,10000.)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,76,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
}

{ /* invalid parameter object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   double stp[] = {1.,2.,3.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,85,(unur_hinv_set_order( par, 3 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,85,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,88,(unur_hinv_set_u_resolution( par, 1.e-8 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,88,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,91,(unur_hinv_set_cpoints(par,stp,3)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,91,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,94,(unur_hinv_set_boundary(par,1.,3.)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,94,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,97,(unur_hinv_set_guidefactor( par, 2. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,97,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,100,(unur_hinv_set_max_intervals(par,10000)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,100,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;
free(par);
unur_distr_free(distr);
}

{ /* invalid parameters */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   double stp[] = {1.,2.,-3.};
   distr = unur_distr_normal(fpar,2);
   par = unur_hinv_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,111,(unur_hinv_set_order( par, 30 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,111,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,114,(unur_hinv_set_u_resolution( par, -1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,114,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,117,(unur_hinv_set_cpoints(par,stp,3)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,117,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,120,(unur_hinv_set_boundary(par,1.,-3.)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,120,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,123,(unur_hinv_set_boundary(par,1.,UNUR_INFINITY)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,123,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,126,(unur_hinv_set_guidefactor( par, -2. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,126,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,129,(unur_hinv_set_max_intervals(par,0)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,129,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/
/* [get] */

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid generator object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_tdr_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 139,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_zero(TESTLOG,143,(unur_hinv_get_n_intervals(gen)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,143,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/
/* [chg] */

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid generator object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   unur_distr_cont_set_domain(distr,-1.,2.);
   par = unur_hinv_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 155,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,159,(unur_hinv_chg_truncated( gen, 1., 0. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,159,UNUR_ERR_DISTR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_hinv_chg_truncated( gen, -2., 0. );
n_tests_failed += (check_errorcode(TESTLOG,162,UNUR_ERR_DISTR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_hinv_chg_truncated( gen, 0., 10. );
n_tests_failed += (check_errorcode(TESTLOG,165,UNUR_ERR_DISTR_SET)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/
/* [init] */

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/
/* [reinit] */

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/
/* [sample] */

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_GEN   *gen = NULL;
   gen = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_INFINITY(TESTLOG,184,(unur_hinv_eval_approxinvcdf(gen,0.5)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,184,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
unur_free(gen);
}

{ /* invalid generator object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_tdr_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 190,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_INFINITY(TESTLOG,194,(unur_hinv_eval_approxinvcdf(gen,0.5)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,194,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* invalid domain */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_hinv_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 200,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_INFINITY(TESTLOG,204,(unur_hinv_eval_approxinvcdf(gen,1.5)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,204,UNUR_ERR_DOMAIN)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_negINFINITY(TESTLOG,207,(unur_hinv_eval_approxinvcdf(gen,-0.5)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,207,UNUR_ERR_DOMAIN)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* compare clone */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   distr = unur_distr_normal(NULL,0); 
   par = NULL;
   gen = NULL; 


unur_errno = 0;
/* original generator object */
par = unur_hinv_new(distr);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,220,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += (compare_double_sequence_gen(TESTLOG,226,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* compare stringparser */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {3.,4.};
   distr = NULL;
   par = NULL;
   gen = NULL; 


unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
unur_distr_cont_set_domain(distr,2,UNUR_INFINITY);
par = unur_hinv_new(distr);
unur_set_debug(par,1u);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,242,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,4.); domain = 2,infinity & \
  method = hinv; debug = 0x1" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,248,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_normal(NULL,0);
unur_distr_cont_set_domain(distr,2,UNUR_INFINITY);
par = unur_hinv_new(distr);
unur_hinv_set_order(par,1);
unur_hinv_set_max_intervals(par,1000);
unur_hinv_set_u_resolution(par,0.001);
unur_hinv_set_boundary(par,2,5);
unur_set_debug(par,1u);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,261,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal; domain = (2,inf) & \
         method = hinv; order = 1; max_intervals = 1000; u_resolution = 0.001; \
         boundary = (2,5); debug = 0x1" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,268,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/
/* [validate] */

/*---------------------------------------------------------------------------*/

/* [validate] */

void test_validate (void)
{
	UNUR_DISTR *distr[35];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	int rcode;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 35 */
{
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);
}

{
fpm[0] = 0.5;
fpm[1] = 3.;
distr[28] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 0.3;
distr[29] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 0.5;
fpm[1] = 0.4;
distr[30] = unur_distr_beta(fpm,2);
}

{
distr[7] = unur_distr_cauchy(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);
}

{
distr[24] = unur_distr_exponential(NULL,0);
}

{
fpm[0] = 30.;
fpm[1] = -5.;
distr[25] = unur_distr_exponential(fpm,2);
}

{
fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);
}

{
fpm[0] = 0.5;
distr[31] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 0.2;
distr[32] = unur_distr_gamma(fpm,1);
}

{
distr[26] = unur_distr_laplace(NULL,0);
}

{
fpm[0] = -10.;
fpm[1] = 100.;
distr[27] = unur_distr_laplace(fpm,2);
}

{
distr[17] = unur_distr_normal(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);
}

{
fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);
}

{
distr[20] = unur_distr_uniform(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);
}

{
distr[22] = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(distr[22],0.1,1.);
unur_distr_cont_upd_mode(distr[22]);
unur_distr_cont_upd_pdfarea(distr[22]);
}

{
fpm[0] = 3.;
fpm[1] = 4.;
distr[23] = unur_distr_beta(fpm,2);
unur_distr_cont_set_domain(distr[23],-2.,5.);
}

{
distr[33] = unur_distr_cont_new();
unur_distr_cont_set_cdf( distr[33], sin_cdf );
unur_distr_cont_set_pdf( distr[33], sin_pdf );
unur_distr_cont_set_dpdf( distr[33], sin_dpdf );
unur_distr_cont_set_domain( distr[33], -1., 1. );
}

{
distr[34] = unur_distr_cont_new();
unur_distr_cont_set_cdf( distr[34], sin0_cdf );
unur_distr_cont_set_pdf( distr[34], sin0_pdf );
unur_distr_cont_set_dpdf( distr[34], sin0_dpdf );
unur_distr_cont_set_domain( distr[34], -1., 1. );
}

	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 245 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [29] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [30] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[30],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[30],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[30],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[30],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[30],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[30],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[30],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[13],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[13],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[13],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[13],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[13],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[13],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[14],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[14],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[14],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[14],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[14],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[14],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[15],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[15],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[15],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[15],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[15],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[15],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[16],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[16],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[16],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[16],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[16],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[16],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [31] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[31],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[31],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[31],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[31],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[31],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[31],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[31],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [32] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[32],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[32],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[32],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[32],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[32],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[32],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[32],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [33] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[33],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[33],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[33],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[33],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[33],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[33],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [34] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[34],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[34],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[34],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[34],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[34],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[34],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
	unur_distr_free(distr[24]);
	unur_distr_free(distr[25]);
	unur_distr_free(distr[26]);
	unur_distr_free(distr[27]);
	unur_distr_free(distr[28]);
	unur_distr_free(distr[29]);
	unur_distr_free(distr[30]);
	unur_distr_free(distr[31]);
	unur_distr_free(distr[32]);
	unur_distr_free(distr[33]);
	unur_distr_free(distr[34]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" ==> failed] ") : printf(" ==> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/
/* [special] */

void test_special (void)
{
	/* set boolean to FALSE */
	int FAILED = 0;
  




  int samplesize = 10000;
  int order = 3;
  int errorsum = 0;
  double fpar[4];
  double eu = 1.e-8;  /* (maximal) u-error */
  double cpoints[10];     

  UNUR_DISTR *distr;
  UNUR_PAR   *par;
  UNUR_GEN   *gen;

/* start test */
printf("[special "); fflush(stdout);
fprintf(TESTLOG,"\n[special]\n");

  /* test for maximal u-error */
  printf("\ntest maximal u-error"); 
  fprintf(TESTLOG,"\nTest maximal u-error:\n"); 

  for ( eu=1.e-6; eu>1.e-13; eu*=0.01 ) {
    for ( order=(eu<1.e-9 ? 3:1); order<=5; order+=2)
      {   
	distr = unur_distr_normal(NULL,0);
	hinv_error_experiment(distr,NULL,0,eu,order,samplesize,2);
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
	
	distr = unur_distr_cauchy(NULL,0);
	if(!hinv_error_experiment(distr,NULL,0,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
	
	distr = unur_distr_exponential(NULL,0);
	if(!hinv_error_experiment(distr,NULL,0,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
	
	fpar[0]= 5.; 
	distr = unur_distr_gamma(fpar,1);
	if(!hinv_error_experiment(distr,NULL,0,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
	
	fpar[0]= 0.5; 
	distr = unur_distr_gamma(fpar,1);
	if(!hinv_error_experiment(distr,NULL,0,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
	
	fpar[0]=2.; fpar[1]=2.;fpar[2]= 0.; fpar[3]=1.;
	distr = unur_distr_beta(fpar,4);
	if(!hinv_error_experiment(distr,NULL,0,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
	
	fpar[0]=0.3; fpar[1]=3.;fpar[2]= 0.; fpar[3]=1.;
	distr = unur_distr_beta(fpar,4);
	if(!hinv_error_experiment(distr,NULL,0,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");

	distr = unur_distr_cont_new();
	unur_distr_set_name(distr,"sin-example");
	unur_distr_cont_set_cdf( distr, sin_cdf );
	unur_distr_cont_set_pdf( distr, sin_pdf );
	unur_distr_cont_set_dpdf( distr, sin_dpdf );
	unur_distr_cont_set_domain( distr, -1., 1. );
	cpoints[0]= -0.75;
	cpoints[1]= -0.25;
	cpoints[2]=  0.25;
	cpoints[3]=  0.75;
	if(!hinv_error_experiment(distr,cpoints,4,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
	
	distr = unur_distr_cont_new();
	unur_distr_set_name(distr,"sin-0-example");
	unur_distr_cont_set_cdf( distr, sin0_cdf );
	unur_distr_cont_set_pdf( distr, sin0_pdf );
	unur_distr_cont_set_dpdf( distr, sin0_dpdf );
	unur_distr_cont_set_domain( distr, -1., 0.5 );
	cpoints[0]= -0.75;
	cpoints[1]= -0.5;
	cpoints[2]=  0.;
	cpoints[3]=  0.25;
	if(!hinv_error_experiment(distr,cpoints,4,eu,order,samplesize,2)) errorsum++;
	unur_distr_free(distr);
	fprintf(TESTLOG,"\n");
      }	
    }
 
  /* extra test for changeing the domain of the generator object */

  printf("\ntest unur_chg_truncated(): "); 
  fprintf(TESTLOG,"\nTests unur_chg_truncated():\n\n"); 

  printf(" normal");
  
  distr = unur_distr_normal(NULL,0);
  par = unur_hinv_new(distr);
  
  eu=1.e-10; order=3; 
  
  unur_hinv_set_order(par,order);
  unur_hinv_set_u_resolution(par,eu);
  gen = unur_init(par);
  
  if(!chg_domain_experiment(gen,eu,order,samplesize,2, -6., 6.)) errorsum++;

  unur_free(gen);
  unur_distr_free(distr);
  fprintf(TESTLOG,"\n");

  /****/

  printf(" sinus1");
  distr = unur_distr_cont_new();
  unur_distr_set_name(distr,"sin-example");
  unur_distr_cont_set_cdf( distr, sin_cdf );
  unur_distr_cont_set_pdf( distr, sin_pdf );
  unur_distr_cont_set_dpdf( distr, sin_dpdf );
  unur_distr_cont_set_domain( distr, -1., 1. );
  cpoints[0]= -0.75;
  cpoints[1]= -0.25;
  cpoints[2]=  0.25;
  cpoints[3]=  0.75;
  par = unur_hinv_new(distr);
  unur_hinv_set_order(par,order);
  unur_hinv_set_cpoints(par, cpoints, 4);
  unur_hinv_set_u_resolution(par,eu);
  gen = unur_init(par);

  if(!chg_domain_experiment(gen,eu,order,samplesize,2, -1., 1.)) errorsum++;

  unur_distr_free(distr);
  unur_free(gen);
  fprintf(TESTLOG,"\n");

  /****/

  printf(" sinus2");
  distr = unur_distr_cont_new();
  unur_distr_set_name(distr,"sin-0-example");
  unur_distr_cont_set_cdf( distr, sin0_cdf );
  unur_distr_cont_set_pdf( distr, sin0_pdf );
  unur_distr_cont_set_dpdf( distr, sin0_dpdf );
  unur_distr_cont_set_domain( distr, -1., 0.5 );
  cpoints[0]= -0.75;
  cpoints[1]= -0.5;
  cpoints[2]=  0.;
  cpoints[3]=  0.25;
  par = unur_hinv_new(distr);
  unur_hinv_set_order(par,order);
  unur_hinv_set_cpoints(par, cpoints, 4);
  unur_hinv_set_u_resolution(par,eu);
  gen = unur_init(par);

  if(!chg_domain_experiment(gen,eu,order,samplesize,2, -1., 0.5)) errorsum++;

  unur_distr_free(distr);
  unur_free(gen);
  fprintf(TESTLOG,"\n");

  /* test finished */
  FAILED = (errorsum == 0) ? 0 : 1;





	/* test finished */
	test_ok &= (FAILED) ? 0 : 1;
	(FAILED) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = special  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_special() */


/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_hinv_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_hinv_set_verify(UNUR_PAR *par, int verify) {return 0;}

/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_hinv_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_hinv_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(346156);
	unur_urng_fish_seed(346156);
	unur_urng_mstd_seed(346156);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(346156)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {346156,346156,346156,346156,346156,346156};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("hinv: ");

	/* run tests */
test_new();
test_set();
test_get();
test_chg();
test_init();
test_reinit();
test_sample();
test_validate();
test_special();


	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? EXIT_SUCCESS : EXIT_FAILURE );

} /* end of main */

