/*
	file automatically generated by make_test_files.pl
	Fri Aug  6 16:51:04 2004
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for NROU
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_nrou_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/



/* prototypes */
double pdf( double x, const UNUR_DISTR *distr );

double pdf_sqrtlin( double x, const UNUR_DISTR *distr );
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr );
double cdf_sqrtlin( double x, const UNUR_DISTR *distr );

double pdf_sqrtlinshft(double x, const UNUR_DISTR *distr );
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr );
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr );


int unur_nrou_set_pedantic( struct unur_par *par, int pedantic );

#define COMPARE_SAMPLE_SIZE   (10000)
#define VIOLATE_SAMPLE_SIZE   (50)




/*---------------------------------------------------------------------------*/
/* [verbatim] */



/* pdf that does not work */
double pdf( double x, const UNUR_DISTR *distr )
{ 
  return ((x==0.) ? 0. : exp(-x*x/2.));
} /* end of pdf */

/* pdf with piecewise linear function as transformed density with T = -1/sqrt */
double pdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}

/* pdf with piecewise linear function as transformed density with T = -1/sqrt and shifted mode */
double pdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	x -= 1000.;
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}

/* dummy function */
int unur_nrou_set_pedantic( struct unur_par *par, int pedantic )
{ return 1; }

/*---------------------------------------------------------------------------*/
/* [new] */

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_DISTR *distr = NULL;
   distr = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,38,(unur_nrou_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,38,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* invalid distribution type */
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,44,(unur_nrou_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,44,UNUR_ERR_DISTR_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* data missing in distribution object */
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 


unur_errno = 0;
/* pdf */
n_tests_failed += (check_expected_NULL(TESTLOG,51,(unur_nrou_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,51,UNUR_ERR_DISTR_REQUIRED)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/
/* [set] */

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_PAR   *par = NULL;
   par = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,61,(unur_nrou_set_u(par,-1,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,61,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,64,(unur_nrou_set_v(par,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,64,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,67,(unur_nrou_set_center(par,0.)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,67,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,70,(unur_nrou_set_verify(par,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,70,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
}

{ /* invalid parameter object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,79,(unur_nrou_set_u(par,-1,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,79,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,82,(unur_nrou_set_v(par,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,82,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,85,(unur_nrou_set_center(par,0.)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,85,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,88,(unur_nrou_set_verify(par,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,88,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;
free(par);
unur_distr_free(distr);
}

{ /* invalid parameters */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_nrou_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,98,(unur_nrou_set_u(par,-1,-2)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,98,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,101,(unur_nrou_set_v(par,0)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,101,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/
/* [chg] */

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid generator object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 117,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,121,(unur_nrou_chg_verify(gen,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,121,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/
/* [init] */

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_PAR   *par = NULL;
   par = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,131,(unur_init( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,131,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/
/* [sample] */

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* compare */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = unur_distr_normal(NULL,0); 
   par = NULL; 


unur_errno = 0;
/* default algorithm */
par = unur_nrou_new(distr);
unur_nrou_set_u(par,-0.56,0.56);
unur_nrou_set_v(par,0.64);
n_tests_failed += (compare_double_sequence_par_start(TESTLOG,150,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* default algorithm - verifying mode */
par = unur_nrou_new(distr);
unur_nrou_set_u(par,-0.56,0.56);
unur_nrou_set_v(par,0.64);
unur_nrou_set_verify(par,1);
n_tests_failed += (compare_double_sequence_par(TESTLOG,157,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* violate condition */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = unur_distr_normal(NULL,0);
   par = NULL; 


unur_errno = 0;
/* v-value is too small */
par = unur_nrou_new(distr);
unur_nrou_set_u(par,-0.56,0.56);
unur_nrou_set_v(par,0.1);
 run_verify_generator (TESTLOG,169,par);
n_tests_failed += (check_errorcode(TESTLOG,169,UNUR_ERR_GEN_CONDITION)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* u-value is too small */
par = unur_nrou_new(distr);
unur_nrou_set_u(par,-0.2,0.56);
unur_nrou_set_v(par,0.64);
 run_verify_generator (TESTLOG,175,par);
n_tests_failed += (check_errorcode(TESTLOG,175,UNUR_ERR_GEN_CONDITION)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* u-value is too small */
par = unur_nrou_new(distr);
unur_nrou_set_u(par,-0.56,0.26);
unur_nrou_set_v(par,0.64);
 run_verify_generator (TESTLOG,181,par);
n_tests_failed += (check_errorcode(TESTLOG,181,UNUR_ERR_GEN_CONDITION)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* compare clone */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   distr = unur_distr_normal(NULL,0); 
   par = NULL;
   gen = NULL; 


unur_errno = 0;
/* original generator object */
par = unur_nrou_new(distr);
unur_nrou_set_u(par,-0.56,0.56);
unur_nrou_set_v(par,0.64);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,196,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += (compare_double_sequence_gen(TESTLOG,202,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* compare stringparser */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = NULL;
   par = NULL;
   gen = NULL; 


unur_errno = 0;
distr = unur_distr_normal(NULL,0);
par = unur_nrou_new(distr);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,214,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal() & method = nrou" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,219,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_normal(NULL,0);
par = unur_nrou_new(distr);
unur_nrou_set_u(par,-0.6,0.6);
unur_nrou_set_v(par,1.);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,227,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal() & method = nrou; u=(-0.6,0.6); v=1" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,232,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/
/* [validate] */

/*---------------------------------------------------------------------------*/

/* [validate] */

void test_validate (void)
{
	UNUR_DISTR *distr[30];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	int rcode;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 30 */
{
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);
}

{
distr[7] = unur_distr_cauchy(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);
}

{
distr[23] = unur_distr_exponential(NULL,0);
}

{
fpm[0] = 30.;
fpm[1] = -5.;
distr[24] = unur_distr_exponential(fpm,2);
}

{
fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);
}

{
fpm[0] = 1.;
fpm[1] = 4.;
distr[29] = unur_distr_gamma(fpm,2);
}

{
distr[25] = unur_distr_laplace(NULL,0);
}

{
fpm[0] = -10.;
fpm[1] = 100.;
distr[26] = unur_distr_laplace(fpm,2);
}

{
distr[17] = unur_distr_normal(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);
}

{
fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);
}

{
distr[20] = unur_distr_uniform(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);
}

{
distr[27] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[27],pdf_sqrtlin);
unur_distr_cont_set_dpdf(distr[27],dpdf_sqrtlin);
unur_distr_cont_set_cdf(distr[27],cdf_sqrtlin);
unur_distr_set_name(distr[27],"sqrtlin");
unur_distr_cont_set_mode(distr[27],0.);
unur_distr_cont_set_pdfarea(distr[27],2.);
}

{
distr[28] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[28],pdf_sqrtlinshft);
unur_distr_cont_set_dpdf(distr[28],dpdf_sqrtlinshft);
unur_distr_cont_set_cdf(distr[28],cdf_sqrtlinshft);
unur_distr_set_name(distr[28],"sqrtlin");
unur_distr_cont_set_mode(distr[28],1000.);
unur_distr_cont_set_pdfarea(distr[28],2.);
}

{
distr[22] = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(distr[22],0.1,1.);
unur_distr_cont_upd_mode(distr[22]);
unur_distr_cont_upd_pdfarea(distr[22]);
}

	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 60 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[0]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[1]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[2]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[3]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[4]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[5]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[6]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[23]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[24]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[7]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[8]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[9]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[10]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[11]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[12]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[13]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[13],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [29] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[29]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[14]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[14],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[15]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[15],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[16]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[16],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[25]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[26]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[26]);
unur_nrou_set_u(par,-0.6,0.6);
unur_nrou_set_v(par,1.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[17]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[17]);
unur_nrou_set_u(par,-0.6,0.6);
unur_nrou_set_v(par,1.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[18]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[19]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[20]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[21]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[22]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[27]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[28]);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("\n(verify hat) "); fflush(stdout);

/* verify hat tests: 60 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nVerify Hat Test (squeeze <= PDF <= hat):\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[0]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[1]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[2]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[3]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[4]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[5]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[6]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[23]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[24]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[7]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[8]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[9]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[10]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[11]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[12]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[13]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[13],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [29] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[29]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[29],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[14]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[14],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[15]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[15],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[16]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[16],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[25]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[26]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[26]);
	unur_nrou_set_pedantic(par,0);
unur_nrou_set_u(par,-0.6,0.6);
unur_nrou_set_v(par,1.);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[17]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[17]);
	unur_nrou_set_pedantic(par,0);
unur_nrou_set_u(par,-0.6,0.6);
unur_nrou_set_v(par,1.);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[18]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[19]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[20]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[21]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[22]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[27]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_nrou_new(distr[28]);
	unur_nrou_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_nrou_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
	unur_distr_free(distr[24]);
	unur_distr_free(distr[25]);
	unur_distr_free(distr[26]);
	unur_distr_free(distr[27]);
	unur_distr_free(distr[28]);
	unur_distr_free(distr[29]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" ==> failed] ") : printf(" ==> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */


/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_nrou_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */


/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_nrou_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_nrou_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(193005);
	unur_urng_fish_seed(193005);
	unur_urng_mstd_seed(193005);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(193005)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {193005,193005,193005,193005,193005,193005};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("nrou: ");

	/* run tests */
test_new();
test_set();
test_chg();
test_init();
test_sample();
test_validate();


	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? EXIT_SUCCESS : EXIT_FAILURE );

} /* end of main */

