/*
	file automatically generated by make_test_files.pl
	Fri Aug  6 16:51:24 2004
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for TDR
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_tdr_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/



/* prototypes */
double pdf( double x, const UNUR_DISTR *distr );

double pdf_bimodal( double x, const UNUR_DISTR *distr );
double dpdf_bimodal( double x, const UNUR_DISTR *distr );

double pdf_negative( double x, const UNUR_DISTR *distr );
double dpdf_negative( double x, const UNUR_DISTR *distr );

double pdf_partnegative( double x, const UNUR_DISTR *distr );
double dpdf_partnegative( double x, const UNUR_DISTR *distr );

double pdf_sqrtlin( double x, const UNUR_DISTR *distr );
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr );
double cdf_sqrtlin( double x, const UNUR_DISTR *distr );

double pdf_sqrtlinshft(double x, const UNUR_DISTR *distr );
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr );
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr );

#define COMPARE_SAMPLE_SIZE   (10000)
#define VIOLATE_SAMPLE_SIZE   (20)




/*---------------------------------------------------------------------------*/
/* [verbatim] */



/* pdf of bimodal density */
double pdf_bimodal( double x, const UNUR_DISTR *distr )
{ 
	return ( exp(-(x-1.)*(x-1.)) + exp(-(x+1.)*(x+1.)) );
}
double dpdf_bimodal( double x, const UNUR_DISTR *distr )
{ 
	return ( -2. * (x-1.) * exp(-(x-1.)*(x-1.)) -2. * (x+1.) * exp(-(x+1.)*(x+1.)) );
}

/* pdf with negative value */
double pdf_negative( double x, const UNUR_DISTR *distr )  { return (-x*x); }
double dpdf_negative( double x, const UNUR_DISTR *distr ) { return (-2.*x); }

/* pdf with partial negative value */
double pdf_partnegative( double x, const UNUR_DISTR *distr )  
{ 
	return ((x>-0.89 && x<0.89) ? -1.: exp(-x*x)); 
}
double dpdf_partnegative( double x, const UNUR_DISTR *distr ) 
{
	return ((x>-0.89 && x<0.89) ?0.: -2.*x*exp(-x*x));
}

/* pdf of normal density */
double pdf( double x, const UNUR_DISTR *distr )
{ 
	return exp(-x*x/2.);
} /* end of pdf */

/* pdf with piecewise linear function as transformed density with T = -1/sqrt */
double pdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}

/* pdf with piecewise linear function as transformed density with T = -1/sqrt and shifted mode */
double pdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	x -= 1000.;
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}


/*---------------------------------------------------------------------------*/
/* [new] */

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_DISTR *distr = NULL;
   distr = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,44,(unur_tdr_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,44,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* invalid distribution type */
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,50,(unur_tdr_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,50,UNUR_ERR_DISTR_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* data missing in distribution object */
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 


unur_errno = 0;
/* pdf, dpdf */
n_tests_failed += (check_expected_NULL(TESTLOG,57,(unur_tdr_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,57,UNUR_ERR_DISTR_REQUIRED)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* dpdf */
unur_distr_cont_set_pdf(distr,pdf);
n_tests_failed += (check_expected_NULL(TESTLOG,62,(unur_tdr_new( distr )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,62,UNUR_ERR_DISTR_REQUIRED)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/
/* [set] */

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_PAR   *par = NULL;
   par = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,72,(unur_tdr_set_cpoints( par, 0, NULL )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,72,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,75,(unur_tdr_set_guidefactor( par, 1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,75,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,78,(unur_tdr_set_max_sqhratio( par, 0.95 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,78,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,81,(unur_tdr_set_max_intervals( par, 100 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,81,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,84,(unur_tdr_set_center( par, 0. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,84,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,87,(unur_tdr_set_usecenter( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,87,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,90,(unur_tdr_set_usemode( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,90,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,93,(unur_tdr_set_variant_gw( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,93,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,96,(unur_tdr_set_variant_ps( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,96,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,99,(unur_tdr_set_variant_ia( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,99,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,102,(unur_tdr_set_usedars( par, TRUE )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,102,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,105,(unur_tdr_set_darsfactor( par, 1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,105,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,108,(unur_tdr_set_c( par, -0.5 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,108,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,111,(unur_tdr_set_verify( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,111,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,114,(unur_tdr_set_pedantic( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,114,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
}

{ /* invalid parameter object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,123,(unur_tdr_set_cpoints( par, 0, NULL )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,123,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,126,(unur_tdr_set_guidefactor( par, 1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,126,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,129,(unur_tdr_set_max_sqhratio( par, 0.95 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,129,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,132,(unur_tdr_set_max_intervals( par, 100 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,132,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,135,(unur_tdr_set_center( par, 0. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,135,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,138,(unur_tdr_set_usecenter( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,138,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,141,(unur_tdr_set_usemode( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,141,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,144,(unur_tdr_set_variant_gw( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,144,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,147,(unur_tdr_set_variant_ps( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,147,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,150,(unur_tdr_set_variant_ia( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,150,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,153,(unur_tdr_set_usedars( par, TRUE )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,153,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,156,(unur_tdr_set_darsfactor( par, 1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,156,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,159,(unur_tdr_set_c( par, -0.5 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,159,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,162,(unur_tdr_set_verify( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,162,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,165,(unur_tdr_set_pedantic( par, 1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,165,UNUR_ERR_PAR_INVALID)==UNUR_SUCCESS)?0:1;
free(par);
unur_distr_free(distr);
}

{ /* invalid parameters */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double stp[] = {1.,0.,1.};
   int n_stp = 3;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_tdr_new(distr); 


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,177,(unur_tdr_set_cpoints( par, -1, NULL )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,177,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,180,(unur_tdr_set_cpoints( par, n_stp, stp )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,180,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,183,(unur_tdr_set_guidefactor( par, -1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,183,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,186,(unur_tdr_set_max_sqhratio( par, -1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,186,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,189,(unur_tdr_set_max_sqhratio( par, 2. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,189,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,192,(unur_tdr_set_max_intervals( par, 0 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,192,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,195,(unur_tdr_set_usedars( par, -1 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,195,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,198,(unur_tdr_set_darsfactor( par, -1. )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,198,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,201,(unur_tdr_set_c( par, 0.5 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,201,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,204,(unur_tdr_set_c( par, -0.6 )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,204,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_tdr_set_c( par, -0.3 );
n_tests_failed += (check_errorcode(TESTLOG,207,UNUR_ERR_PAR_SET)==UNUR_SUCCESS)?0:1;
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/
/* [get] */

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid generator object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_arou_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 217,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,221,(unur_tdr_get_sqhratio(gen)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,221,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,224,(unur_tdr_get_hatarea(gen)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,224,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,227,(unur_tdr_get_squeezearea(gen)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,227,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/
/* [chg] */

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid generator object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 238,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_setfailed(TESTLOG,242,(unur_tdr_chg_verify(gen,1)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,242,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/
/* [init] */

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_PAR   *par = NULL;
   par = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,253,(unur_init( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,253,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
}

{ /* stp out of domain */
UNUR_GEN   *gen = NULL;
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double stp[] = {-2.5,-1.,0.,1.,2.5};
   int n_stp = 5;
   gen = NULL;
   distr = unur_distr_normal(NULL,0);
   unur_distr_cont_set_domain(distr,-2.,2.);
   par = unur_tdr_new(distr);
   unur_tdr_set_usedars(par,FALSE);
   unur_tdr_set_cpoints( par, n_stp, stp ); 


unur_errno = 0;
gen = unur_init( par );
n_tests_failed += (check_errorcode(TESTLOG,266,UNUR_ERR_GEN_DATA)==UNUR_SUCCESS)?0:1;
unur_free(gen);
unur_distr_free(distr);
}

{ /* bimodal */
UNUR_GEN   *gen = NULL;
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   gen = NULL;
   distr = unur_distr_cont_new(); 
   unur_distr_cont_set_pdf(distr,pdf_bimodal);
   unur_distr_cont_set_dpdf(distr,dpdf_bimodal);
   par = unur_tdr_new(distr);
   unur_tdr_set_usedars(par,FALSE);
   unur_tdr_set_cpoints( par, 30, NULL ); 


unur_errno = 0;
n_tests_failed += (check_expected_NULL(TESTLOG,278,(gen = unur_init( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,278,UNUR_ERR_GEN_CONDITION)==UNUR_SUCCESS)?0:1;
unur_free(gen);
unur_distr_free(distr);
}

{ /* wrong mode */
UNUR_GEN   *gen = NULL;
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = NULL; 


unur_errno = 0;
unur_distr_cont_set_mode(distr,-1.);
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
n_tests_failed += (check_expected_NULL(TESTLOG,290,(gen = unur_init( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,290,UNUR_ERR_GEN_DATA)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_distr_cont_set_mode(distr,1.);
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
gen = unur_init( par );
n_tests_failed += (check_errorcode(TESTLOG,297,UNUR_ERR_GEN_DATA)==UNUR_SUCCESS)?0:1;
unur_free(gen);
unur_distr_free(distr);
}

{ /* negative value of pdf */
UNUR_GEN   *gen = NULL;
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   gen = NULL;
   distr = unur_distr_cont_new(); 
   unur_distr_cont_set_pdf(distr,pdf_negative);
   unur_distr_cont_set_dpdf(distr,dpdf_negative);
   par = NULL; 


unur_errno = 0;
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
n_tests_failed += (check_expected_NULL(TESTLOG,310,(gen = unur_init( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,310,UNUR_ERR_GEN_DATA)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_distr_cont_set_domain(distr,-2.,2.);
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
n_tests_failed += (check_expected_NULL(TESTLOG,317,(gen = unur_init( par )))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,317,UNUR_ERR_GEN_DATA)==UNUR_SUCCESS)?0:1;
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/
/* [reinit] */

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/
/* [sample] */

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
{ /* invalid NULL ptr */
UNUR_GEN   *gen = NULL;
   gen = NULL; 


unur_errno = 0;
n_tests_failed += (check_expected_INFINITY(TESTLOG,331,(unur_tdr_eval_invcdfhat(gen,0.5,NULL,NULL,NULL)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,331,UNUR_ERR_NULL)==UNUR_SUCCESS)?0:1;
unur_free(gen);
}

{ /* invalid generator object */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_arou_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 337,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_INFINITY(TESTLOG,341,(unur_tdr_eval_invcdfhat(gen,0.5,NULL,NULL,NULL)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,341,UNUR_ERR_GEN_INVALID)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* invalid domain */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_tdr_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 348,    gen );


unur_errno = 0;
n_tests_failed += (check_expected_INFINITY(TESTLOG,352,(unur_tdr_eval_invcdfhat(gen,1.5,NULL,NULL,NULL)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,352,UNUR_ERR_DOMAIN)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
n_tests_failed += (check_expected_negINFINITY(TESTLOG,355,(unur_tdr_eval_invcdfhat(gen,-0.5,NULL,NULL,NULL)))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,355,UNUR_ERR_DOMAIN)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* partial negative value of pdf */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   int i;
   double x;
   double stp[] = {-0.9,0.9};
   int n_stp = 2;
   distr = unur_distr_cont_new(); 
   unur_distr_cont_set_pdf(distr,pdf_partnegative);
   unur_distr_cont_set_dpdf(distr,dpdf_partnegative);
   par = unur_tdr_new(distr);
   unur_tdr_set_usedars(par,FALSE);
   unur_tdr_set_cpoints( par, n_stp, stp );
   unur_tdr_set_pedantic( par, 1 );
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 370,    gen );


unur_errno = 0;
for (i=0; i<100; i++) { x = unur_sample_cont(gen); }
n_tests_failed += (check_expected_INFINITY(TESTLOG,375,(x))==UNUR_SUCCESS)?0:1;
n_tests_failed += (check_errorcode(TESTLOG,375,UNUR_ERR_GEN_CONDITION)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* compare */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = unur_distr_normal(NULL,0); 
   par = NULL; 


unur_errno = 0;
/* GW, c = -0.5 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par_start(TESTLOG,390,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* GW, c = -0.5 - verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par(TESTLOG,398,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* PS, c = -0.5 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par_start(TESTLOG,405,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* PS, c = -0.5 - verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par(TESTLOG,413,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* IA, c = -0.5 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par_start(TESTLOG,420,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* IA, c = -0.5 - verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par(TESTLOG,428,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* GW, c = 0 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par_start(TESTLOG,438,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* GW, c = 0, verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par(TESTLOG,446,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* PS, c = 0 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par_start(TESTLOG,453,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* PS, c = 0, verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par(TESTLOG,461,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* IA, c = 0 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par_start(TESTLOG,468,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* IA, c = 0, verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += (compare_double_sequence_par(TESTLOG,476,par,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
}

{ /* compare clone */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   distr = unur_distr_normal(NULL,0); 
   par = NULL;
   gen = NULL; 


unur_errno = 0;
/* original generator object */
par = unur_tdr_new(distr);
unur_tabl_set_max_sqhratio(par,0.);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,490,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += (compare_double_sequence_gen(TESTLOG,496,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}

{ /* compare stringparser */
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 


unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_center(par,2.1);
unur_tdr_set_cpoints(par,20,NULL);
unur_tdr_set_guidefactor(par,2.);
unur_tdr_set_max_intervals(par,32);
unur_tdr_set_max_sqhratio(par,0.9);
unur_tdr_set_usecenter(par,TRUE);
unur_tdr_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,516,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; center = 2.1; usecenter; cpoints = 20; guidefactor = 2.; \
  max_intervals = 32; max_sqhratio = 0.9; verify" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,523,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_cpoints(par,9,NULL);
unur_tdr_set_pedantic(par,TRUE);
unur_tdr_set_usemode(par,FALSE);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,533,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; cpoints = 9; pedantic; usemode = false" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,539,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_c(par,0.);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,547,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; c = 0." );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,553,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_c(par,-0.5);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,561,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; c = -0.5" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,567,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,575,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_gw" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,581,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,589,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_ia" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,595,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
gen = unur_init(par);
n_tests_failed += (compare_double_sequence_gen_start(TESTLOG,603,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_ps" );
n_tests_failed += (compare_double_sequence_gen(TESTLOG,609,gen,COMPARE_SAMPLE_SIZE)==UNUR_SUCCESS)?0:1;
unur_distr_free(distr);
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" ==> failed] ") : printf(" ==> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/
/* [validate] */

/*---------------------------------------------------------------------------*/

/* [validate] */

void test_validate (void)
{
	UNUR_DISTR *distr[30];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	int rcode;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 30 */
{
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);
}

{
distr[7] = unur_distr_cauchy(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);
}

{
distr[24] = unur_distr_exponential(NULL,0);
}

{
fpm[0] = 30.;
fpm[1] = -5.;
distr[25] = unur_distr_exponential(fpm,2);
}

{
fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);
}

{
fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);
}

{
fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);
}

{
distr[26] = unur_distr_laplace(NULL,0);
}

{
fpm[0] = -10.;
fpm[1] = 100.;
distr[27] = unur_distr_laplace(fpm,2);
}

{
distr[17] = unur_distr_normal(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);
}

{
fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);
}

{
distr[20] = unur_distr_uniform(NULL,0);
}

{
fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);
}

{
distr[28] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[28],pdf_sqrtlin);
unur_distr_cont_set_dpdf(distr[28],dpdf_sqrtlin);
unur_distr_cont_set_cdf(distr[28],cdf_sqrtlin);
unur_distr_set_name(distr[28],"sqrtlin");
}

{
distr[29] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[29],pdf_sqrtlinshft);
unur_distr_cont_set_dpdf(distr[29],dpdf_sqrtlinshft);
unur_distr_cont_set_cdf(distr[29],cdf_sqrtlinshft);
unur_distr_set_name(distr[29],"sqrtlin");
}

{
distr[22] = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(distr[22],0.1,1.);
unur_distr_cont_upd_mode(distr[22]);
unur_distr_cont_upd_pdfarea(distr[22]);
}

{
fpm[0] = 3.;
fpm[1] = 4.;
distr[23] = unur_distr_beta(fpm,2);
unur_distr_cont_set_domain(distr[23],-2.,5.);
}

	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 570 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[0],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[1],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[2],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[3],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[4],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[5],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[6],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[7],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'-');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[8],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[24],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[25],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[9],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[10],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[11],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[12],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[26],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[27],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[17],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[18],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[19],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[20],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[21],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[22],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[23],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[28],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

/* distribution [29] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'0');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	rcode = run_validate_chi2(TESTLOG,0,gen,distr[29],'+');
	n_tests_failed += (rcode==UNUR_SUCCESS)?0:1;
	n_tests_failed += (rcode==UNUR_FAILURE)?10:0;
	unur_free(gen);
	} while (0);

	printf("\n(verify hat) "); fflush(stdout);

/* verify hat tests: 570 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nVerify Hat Test (squeeze <= PDF <= hat):\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[0],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[1],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[2],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[3],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[4],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[5],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[6],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'0')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'0')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'0')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[7],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'0')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'0')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'-')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'0')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[8],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[24],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[25],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[9],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[10],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[11],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[12],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[26],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[27],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[17],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[18],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[19],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[20],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[21],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[22],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[23],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += (run_validate_verifyhat(TESTLOG,0,gen,distr[28],'~')==UNUR_SUCCESS)?0:2;
	unur_free(gen);

	} while (0);

/* distribution [29] */

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
	unur_distr_free(distr[24]);
	unur_distr_free(distr[25]);
	unur_distr_free(distr[26]);
	unur_distr_free(distr[27]);
	unur_distr_free(distr[28]);
	unur_distr_free(distr[29]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" ==> failed] ") : printf(" ==> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */


/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_tdr_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */


/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_tdr_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_tdr_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(616988);
	unur_urng_fish_seed(616988);
	unur_urng_mstd_seed(616988);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(616988)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {616988,616988,616988,616988,616988,616988};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("tdr: ");

	/* run tests */
test_new();
test_set();
test_get();
test_chg();
test_init();
test_reinit();
test_sample();
test_validate();


	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? EXIT_SUCCESS : EXIT_FAILURE );

} /* end of main */

